@@ -7,11 +7,14 @@ set.seed(1014)
 
 ## ----setup--------------------------------------------------------------------
 library(vctrs)
+library(rlang)
 library(zeallot)
 
 ## -----------------------------------------------------------------------------
 new_percent <- function(x = double()) {
-  vec_assert(x, double())
+  if (!is_double(x)) {
+    abort("`x` must be a double vector.")
+  }
   new_vctr(x, class = "vctrs_percent")
 }
 
@@ -132,8 +135,13 @@ as_percent.character <- function(x) {
 
 ## -----------------------------------------------------------------------------
 new_decimal <- function(x = double(), digits = 2L) {
-  vec_assert(x, ptype = double())
-  vec_assert(digits, ptype = integer(), size = 1)
+  if (!is_double(x)) {
+    abort("`x` must be a double vector.")
+  }
+  if (!is_integer(digits)) {
+    abort("`digits` must be an integer vector.")
+  }
+  vec_check_size(digits, size = 1L)
 
   new_vctr(x, digits = digits, class = "vctrs_decimal")
 }
@@ -196,8 +204,13 @@ vec_cast(c(1.5, 2, 10.5), to = integer())
 
 ## -----------------------------------------------------------------------------
 new_cached_sum <- function(x = double(), sum = 0L) {
-  vec_assert(x, ptype = double())
-  vec_assert(sum, ptype = double(), size = 1L)
+  if (!is_double(x)) {
+    abort("`x` must be a double vector.")
+  }
+  if (!is_double(sum)) {
+    abort("`sum` must be a double vector.")
+  }
+  vec_check_size(sum, size = 1L)
 
   new_vctr(x, sum = sum, class = "vctrs_cached_sum")
 }
@@ -249,8 +262,12 @@ unclass(x)[[1]] # the first component, the number of seconds
 
 ## -----------------------------------------------------------------------------
 new_rational <- function(n = integer(), d = integer()) {
-  vec_assert(n, ptype = integer())
-  vec_assert(d, ptype = integer())
+  if (!is_integer(n)) {
+    abort("`n` must be an integer vector.")
+  }
+  if (!is_integer(d)) {
+    abort("`d` must be an integer vector.")
+  }
 
   new_rcrd(list(n = n, d = d), class = "vctrs_rational")
 }
@@ -315,9 +332,16 @@ vec_c(rational(1, 2), 1L, NA)
 
 ## -----------------------------------------------------------------------------
 new_decimal2 <- function(l, r, scale = 2L) {
-  vec_assert(l, ptype = integer())
-  vec_assert(r, ptype = integer())
-  vec_assert(scale, ptype = integer(), size = 1L)
+  if (!is_integer(l)) {
+    abort("`l` must be an integer vector.")
+  }
+  if (!is_integer(r)) {
+    abort("`r` must be an integer vector.")
+  }
+  if (!is_integer(scale)) {
+    abort("`scale` must be an integer vector.")
+  }
+  vec_check_size(scale, size = 1L)
 
   new_rcrd(list(l = l, r = r), scale = scale, class = "vctrs_decimal2")
 }
@@ -369,34 +393,38 @@ x == rational(1, 1)
 unique(x)
 
 ## -----------------------------------------------------------------------------
-sort(x)
+rational(1, 2) < rational(2, 3)
+rational(2, 4) < rational(2, 3)
 
 ## -----------------------------------------------------------------------------
 vec_proxy_compare.vctrs_rational <- function(x, ...) {
   field(x, "n") / field(x, "d")
 }
 
+rational(2, 4) < rational(2, 3)
+
+## -----------------------------------------------------------------------------
 sort(x)
 
 ## -----------------------------------------------------------------------------
-new_poly <- function(x) {
-  new_list_of(x, ptype = integer(), class = "vctrs_poly")
-}
-
 poly <- function(...) {
-  x <- list(...)
-  x <- lapply(x, vec_cast, integer())
+  x <- vec_cast_common(..., .to = integer())
   new_poly(x)
 }
+new_poly <- function(x) {
+  new_list_of(x, ptype = integer(), class = "vctrs_poly_list")
+}
 
-vec_ptype_full.vctrs_poly <- function(x, ...) "polynomial"
-vec_ptype_abbr.vctrs_poly <- function(x, ...) "poly"
+vec_ptype_full.vctrs_poly_list <- function(x, ...) "polynomial"
+vec_ptype_abbr.vctrs_poly_list <- function(x, ...) "poly"
 
-format.vctrs_poly <- function(x, ...) {
+format.vctrs_poly_list <- function(x, ...) {
   format_one <- function(x) {
     if (length(x) == 0) {
       return("")
-    } else if (length(x) == 1) {
+    }
+
+    if (length(x) == 1) {
       format(x)
     } else {
       suffix <- c(paste0("\u22C5x^", seq(length(x) - 1, 1)), "")
@@ -405,16 +433,17 @@ format.vctrs_poly <- function(x, ...) {
       paste0(out, collapse = " + ")
     }
   }
+
   vapply(x, format_one, character(1))
 }
 
-obj_print_data.vctrs_poly <- function(x, ...) {
-  if (length(x) == 0)
-    return()
-  print(format(x), quote = FALSE)
+obj_print_data.vctrs_poly_list <- function(x, ...) {
+  if (length(x) != 0) {
+    print(format(x), quote = FALSE)
+  }
 }
 
-p <- poly(1, c(1, 0, 1), c(1, 0, 0, 0, 2))
+p <- poly(1, c(1, 0, 0, 0, 2), c(1, 0, 1))
 p
 
 ## -----------------------------------------------------------------------------
@@ -422,15 +451,40 @@ class(p)
 p[2]
 p[[2]]
 
+## -----------------------------------------------------------------------------
+obj_is_list(p)
+
+## -----------------------------------------------------------------------------
+poly <- function(...) {
+  x <- vec_cast_common(..., .to = integer())
+  x <- new_poly(x)
+  new_rcrd(list(data = x), class = "vctrs_poly")
+}
+format.vctrs_poly <- function(x, ...) {
+  format(field(x, "data"))
+}
+
+## -----------------------------------------------------------------------------
+p <- poly(1, c(1, 0, 0, 0, 2), c(1, 0, 1))
+p
+
+## -----------------------------------------------------------------------------
+obj_is_list(p)
+
+## -----------------------------------------------------------------------------
+p[[2]]
+
 ## -----------------------------------------------------------------------------
 p == poly(c(1, 0, 1))
 
 ## ---- error = TRUE------------------------------------------------------------
-sort(p)
+p < p[2]
 
 ## -----------------------------------------------------------------------------
 vec_proxy_compare.vctrs_poly <- function(x, ...) {
-  x_raw <- vec_data(x)
+  # Get the list inside the record vector
+  x_raw <- vec_data(field(x, "data"))
+
   # First figure out the maximum length
   n <- max(vapply(x_raw, length, integer(1)))
 
@@ -441,8 +495,18 @@ vec_proxy_compare.vctrs_poly <- function(x, ...) {
   as.data.frame(do.call(rbind, full))
 }
 
-sort(poly(3, 2, 1))
-sort(poly(1, c(1, 0, 0), c(1, 0)))
+p < p[2]
+
+## -----------------------------------------------------------------------------
+sort(p)
+sort(p[c(1:3, 1:2)])
+
+## -----------------------------------------------------------------------------
+vec_proxy_order.vctrs_poly <- function(x, ...) {
+  vec_proxy_compare(x, ...)
+}
+
+sort(p)
 
 ## -----------------------------------------------------------------------------
 vec_arith.MYCLASS <- function(op, x, y, ...) {
@@ -557,7 +621,9 @@ vec_arith.vctrs_meter.MISSING <- function(op, x, y, ...) {
 
 ## -----------------------------------------------------------------------------
 new_percent <- function(x = double()) {
-  vec_assert(x, double())
+  if (!is_double(x)) {
+    abort("`x` must be a double vector.")
+  }
   new_vctr(x, class = "pizza_percent")
 }
 
@@ -620,6 +686,32 @@ is_percent <- function(x) {
 #  #' @export
 #  vec_cast.double.pizza_percent <- function(x, to, ...) vec_data(x)
 
+## ---- eval=FALSE--------------------------------------------------------------
+#  #' @export
+#  #' @method vec_arith my_type
+#  vec_arith.my_type <- function(op, x, y, ...) {
+#    UseMethod("vec_arith.my_type", y)
+#  }
+
+## ---- eval=FALSE--------------------------------------------------------------
+#  #' @export
+#  #' @method vec_arith.my_type my_type
+#  vec_arith.my_type.my_type <- function(op, x, y, ...) {
+#    # implementation here
+#  }
+#  
+#  #' @export
+#  #' @method vec_arith.my_type integer
+#  vec_arith.my_type.integer <- function(op, x, y, ...) {
+#    # implementation here
+#  }
+#  
+#  #' @export
+#  #' @method vec_arith.integer my_type
+#  vec_arith.integer.my_type <- function(op, x, y, ...) {
+#    # implementation here
+#  }
+
 ## ---- eval = FALSE------------------------------------------------------------
 #  expect_error(vec_c(1, "a"), class = "vctrs_error_incompatible_type")
 
